########################################################################
# tbb-config.cmake
#
# Note that in addition to providing the config information required by
# CMake for Intel's TBB, we also provide two macros. The first,
# "tbb_offload," adds the requisite properties to each provided source
# file to enable mic offloading to be used by Intel's TBB in conjunction
# with the Intel compiler suite. You should use this if you have
# mic-specific code for TBB to use.
#
# Usage: tbb_offload(<source> ...)
#
# The second macro, "find_tbb_offloads," is for the macro implementer
# rather than the user, and will flag whether any sources in the
# provided list are flagged for mic offloading with TBB in order to
# allow the correct flag (${TBB_OFFLOAD_FLAG}) to be added to library
# and exec link lines as appropriate.
#
# Usage: find_tbb_offloads([FOUND_VAR <fv>]
#                          [FOUND_LIST_VAR <flv>]
#                          <source> ...)
#
# Notes:
#
#   FOUND_VAR <fv> specifies that fv should hold the boolean indicating
#   whether any of the provided sources were flagged for TBB offloading.
#
#   FOUND_LIST_VAR <flv> specifies that the flagged source file names
#   should be saved to the specified list variable.
#
#   At least one of the two aforementioned key / value pairs should be
#   provided to find_tbb_offloads.
#
#   If the Intel compiler suite is in use, TBB_OFFLOAD_FLAG will hold
#   "-tbb," the flag required to be added to the invocation of the
#   compiler.
########################################################################
set( tbb_VERSION 4.2.3 )
set( tbb_UPS_VERSION v4_2_3 )

# Note this -config.cmake file provides two macro, "tbb_offload":
#   tbb_offload(<source-file>...)
# which (when used in conjunction with the intel compiler set) will
# instruct the compiler to find the headers and do the necessary
# cross-compilation for the mic architecture.


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../../../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach(comp)
endmacro()

####################################################################################


get_filename_component(CMAKE_CXX_COMPILER_NAME
  ${CMAKE_CXX_COMPILER}
  NAME
  )

function(tbb_offload)
  if (TBB_OFFLOAD_FLAG AND ARGN AND (CMAKE_CXX_COMPILER_NAME STREQUAL "icpc"))
    set_property(SOURCE ${ARGN}
      APPEND_STRING
      PROPERTY
      COMPILE_FLAGS " ${TBB_OFFLOAD_FLAG}"
      )
  endif()
endfunction()

include(CMakeParseArguments)

function(find_tbb_offloads)
  CMAKE_PARSE_ARGUMENTS("FTBB" "" "FOUND_VAR;FOUND_LIST_VAR" "" ${ARGN})
  if((NOT FTBB_FOUND_VAR) AND (NOT FTBB_FOUND_LIST_VAR))
    message(FATAL_ERROR "find_tbb_offloads expects a boolean result variable FOUND_VAR\n"
      "and/or a list FOUND_LIST_VAR into which to store the found source file names.")
  endif()
  if(FTBB_FOUND_VAR)
    set(${FTBB_FOUND_VAR} FALSE PARENT_SCOPE)
  endif()
  if(NOT TBB_OFFLOAD_FLAG)
    return()
  endif()
  foreach(arg ${FTBB_UNPARSED_ARGUMENTS})
    get_source_file_property(res "${arg}" COMPILE_FLAGS)
    if(res MATCHES "(^| )-tbb( |$)")
      if(FTBB_FOUND_VAR)
        set(${FTBB_FOUND_VAR} TRUE PARENT_SCOPE)
      endif()
      if(FTBB_FOUND_LIST_VAR)
        list(APPEND tbb_offloads "${arg}")
      else()
        return() # Short-circuit success.
      endif()
    endif()
  endforeach()
  if (FTBB_FOUND_LIST_VAR)
    set(${FTBB_FOUND_LIST_VAR} ${have_tbb} PARENT_SCOPE)
  endif()
endfunction()

find_library(TBB NAMES tbb PATHS $ENV{TBB_LIB} NO_DEFAULT_PATH )
find_library(TBBMALLOC NAMES tbbmalloc PATHS $ENV{TBB_LIB} NO_DEFAULT_PATH )
find_library(TBBMALLOC_PROXY NAMES tbbmalloc_proxy PATHS $ENV{TBB_LIB} NO_DEFAULT_PATH )

check_required_components(tbb)

if ((CMAKE_CXX_COMPILER_NAME STREQUAL "icpc") AND tbb_FOUND)
  # Support offloading to mic.
  set(TBB_OFFLOAD_FLAG "-tbb" CACHE STRING "Extra flag required by icpc when offloading using Intel TBB" FORCE)
else()
  unset(TBB_OFFLOAD_FLAG CACHE)
endif()
